import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosEventComm.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import java.util.*;

import easyNotify.*;
import SalleEnchere.*;   // parceque l'vnement envoie l'enchre en cours
import VenteAuxEncheres.*;

/**
 * vnement 
 *
 * cet vnement fait passer l'enchre courante et un message du commissaire priseur (facultatif)
 *
 **/
public class SdvEvent extends EasyStructuredEvent {
    final static String DOMAIN_NAME = "VENTES";
    final static String DOMAIN_TYPE = "Enchere";

    final static String MESG = "Message";
    final static String ENCHR_IDPERS = "IdPers";
    final static String ENCHR_IDPROD = "IdProd";
    final static String ENCHR_PRIXPROD = "PrixProd";
    final static String ENCHR_STATUT = "Statut";
    final static String NOM_PERSONNE = "Nom";

    protected Enchere enchere;
    protected String nomPersonne;
    protected String message;

    public SdvEvent(ORB orb, String nomPersonne, Enchere enchere) {
	this(orb, enchere, nomPersonne, "");
    }

    public SdvEvent(ORB orb, Enchere enchere, String nomPersonne, String message) {
	super(orb, DOMAIN_NAME, DOMAIN_TYPE, enchere+nomPersonne+message);

	// mmorisation des donnes
	this.enchere = enchere;
	this.nomPersonne = nomPersonne;
	this.message = message;

	// positionnement du body  partir des infos de l'enchre.
	Property[] body = new Property[6];
	body[0] = new Property(ENCHR_IDPERS, string2Any(String.valueOf(enchere.idPersonne)));
	body[1] = new Property(ENCHR_IDPROD, string2Any(String.valueOf(enchere.idProduitCourant)));
	body[2] = new Property(ENCHR_PRIXPROD, string2Any(String.valueOf(enchere.prixCourant)));
	body[3] = new Property(ENCHR_STATUT, string2Any(String.valueOf(enchere.leStatut.value())));
	body[4] = new Property(NOM_PERSONNE, string2Any(String.valueOf(nomPersonne)));
	body[5] = new Property(MESG, string2Any(message));
	setBody(body);
    }


    public SdvEvent(StructuredEvent event) throws BadEvent {
	super(event);

	// extraction du body spcifique  cet vnement
	boolean idPersrenseigne = false,   //  true ds que le champ en question a t renseign
	    idProdrenseigne = false,
	    prixCourantrenseigne = false,
	    statutrenseigne = false;

	enchere = new Enchere();
	message = null;
	if (domainName.equals(DOMAIN_NAME) && typeName.equals(DOMAIN_TYPE)) {
	    for (int i = 0 ; i < body.length ; i++) {
		try {
		    if (body[i].name.equals(ENCHR_IDPERS)) {
			this.enchere.idPersonne =  (new Integer(body[i].value.extract_string())).intValue();
			idPersrenseigne = true;
		    } else if (body[i].name.equals(ENCHR_IDPROD)) {
			this.enchere.idProduitCourant = (new Integer(body[i].value.extract_string())).intValue();
			idProdrenseigne = true;
		    } else if (body[i].name.equals(ENCHR_PRIXPROD)) {
			this.enchere.prixCourant = (new Integer(body[i].value.extract_string())).intValue();
			prixCourantrenseigne = true;
		    } else if (body[i].name.equals(ENCHR_STATUT)) {
			// PRECONDITION : le statut a une valeur correcte pour la classe Statut !!!
			this.enchere.leStatut = Statut.from_int((new Integer(body[i].value.extract_string())).intValue());
			statutrenseigne = true;
		    } else if (body[i].name.equals(NOM_PERSONNE)) {
			this.nomPersonne = body[i].value.extract_string();
		    } else if (body[i].name.equals(MESG)) {
			this.message = body[i].value.extract_string();
		    }
		} catch (NumberFormatException e) { // conversion en entier impossible !
		    System.out.println("attention : probleme de conversion en int : " + body[i].name + " = " + body[i].value.extract_string()); 
		}
	    }
	    
	    // mal renseign!
	    if (!(idPersrenseigne && idProdrenseigne && prixCourantrenseigne && statutrenseigne)) {
		System.err.println("Evnement bien typ, mais mal renseign");
		throw new BadEvent();
	    }
	} else {
	    // mauvais type
	    System.err.println("Evnement mal typ");
	    throw new BadEvent();
	}
    }



    /**
     * rcupration des informations
     **/

    public Enchere getEnchere() { return enchere; }

    public String getFieldName() { return nomPersonne; }

    public Statut getFieldStatus() { return (enchere.leStatut); }

    public int getFieldProduit() { return(enchere.idProduitCourant); }

    public int getFieldPrix() { return(enchere.prixCourant); }

    public String getMessage() { return message; }



    /**
     * surcharge du toString pour qu'il affiche bien le body
     **/
    public String filterableBodyToString() {
 	StringBuffer stringTmp = new StringBuffer("");
	/*
 	stringTmp.append("<FilterableBody>");
 	  stringTmp.append("<ENCHERE>");
	    stringTmp.append("<idPersonne>" + enchere.idPersonne + "</idPersonne>");
	    stringTmp.append("<nomPersonne>" + nomPersonne + "</nomPersonne>");
 	    stringTmp.append("<idProduitCourant>" + enchere.idProduitCourant + "</idProduitCourant>");
	    stringTmp.append("<prixCourant>" + enchere.prixCourant + "</prixCourant>");
	    stringTmp.append("<leStatut>" + enchere.leStatut + "</leStatut>");
  	  stringTmp.append("</ENCHERE>");
  	  stringTmp.append("<"+MESG+">"+message+"</"+MESG+">");
  	stringTmp.append("</FilterableBody>");
	*/
	return stringTmp.toString();
    }

}
